@@ -140,9 +140,15 @@ void CollectOperandAfterI(Function &F, Instruction* boundary, Type* T,
 
     if (boundary == NULL) { // Get the all from the function
         for (Instruction &I : instructions(F)) {
-            if (I.getName().find("nop"))
+            if (I.getName().find("nop") != StringRef::npos)
                 continue;
             for (unsigned i=0; i<I.getNumOperands(); i++) {
+                if (isa<CallInst>(I)) {
+                    CallInst* calI = cast<CallInst>(&I);
+                    if (calI->paramHasAttr(i, Attribute::ImmArg))
+                        continue;
+                }
+
                 Value *op = I.getOperand(i);
                 if (T == NULL)
                     OPvec.push_back(std::make_pair(&I, i));
@@ -160,6 +166,14 @@ void CollectOperandAfterI(Function &F, Instruction* boundary, Type* T,
                 continue;
 
             for (unsigned i=0; i<I.getNumOperands(); i++) {
+                // ignore the immediate argument of a call instruction
+                if (isa<CallInst>(I)) {
+                    CallInst* calI = cast<CallInst>(&I);
+                    if (calI->paramHasAttr(i, Attribute::ImmArg))
+                        if (isa<Constant>(boundary) == false)
+                            continue;
+                }
+
                 Value *op = I.getOperand(i);
                 if (op == boundary)
                     continue;
@@ -179,6 +193,7 @@ void CollectOperandAfterI(Function &F, Instruction* boundary, Type* T,
     }
 }
 
+//Note: Type T is implicitely extracted from the boundary instruction
 std::pair<Instruction*, unsigned> randOperandAfterI(Function &F, Instruction* boundary, Type* T) 
 {
     std::vector<std::pair<Instruction*, unsigned>> OPvec;
@@ -577,6 +592,15 @@ int replaceOperands(StringRef dst_desc, StringRef src_desc, Module &M)
             return -3;
         if (DV->getType() != SV->getType())
             return -4;
+        if (isa<CallInst>(DI)) {
+            CallInst* calI = cast<CallInst>(DI);
+            if (calI->paramHasAttr(OPindex, Attribute::ImmArg)) {
+                if (isa<Constant>(SV) == false)
+                    return -6;
+            }
+        }
+        if (dominates(SV, DI) == false)
+            return -7;
         if (DV == SV)
             return 1;
     }
